/*
   Compute Pi using a "Monte Carlo" technique.
      http://en.wikipedia.org/wiki/Monte_Carlo_method
   See the end of this file for some details.

   This is a pthread version of the program. This is essentially
   the program that all the other OpenMP versions will "compile" to.
*/
#include <stdio.h>
#include <windows.h>
#include <math.h>
#include <pthread.h>

// thread function prototype
void *thread(void *vargp);

// global variables
int sample_points_per_thread;
int* sum;      // dynamically allocated array to hold individual thread results

int main(int argc, char** argv)
{
   int i;
   double pi;
   int nthreads;  // number of threads to use
   int npoints;   // number of sample points to generate
   int total_sum = 0;
   pthread_t* tid;   // pointer to dynamically allocated array of thread handles
   int* threadParam; // pointer to dynamically allocated array of thread parameters

   if (argc < 1)
   {
      printf("Usage: %s <number of threads>\n", argv[0]);
      return 0;
   }
   nthreads = strtol(argv[1], NULL, 10);
   npoints = 100000000;  // 100,000,000

   if ( (npoints % nthreads) != 0 )  // nthreads must divide npoints
   {
      printf("number of threads (%d) must divide number of points (%d)\n", nthreads, npoints);
      return 0;
   }
   // compute how much work each thread needs to do
   sample_points_per_thread = npoints / nthreads;

   //srand( GetTickCount() );  // this is a Win32 function
   // allocate the array to hold individual thread results
   sum = (int*)calloc( nthreads, sizeof(int) );

   // allocate the array to hold thread handles
   tid = (pthread_t*)calloc( nthreads, sizeof(pthread_t) );
   // allocate the array to hold thread parameters
   threadParam = (int*)calloc( nthreads, sizeof(int) );

   // create the threads
   for (i = 0; i < nthreads; i++)
   {
      threadParam[i] = i;
      pthread_create(&(tid[i]), NULL, thread, &threadParam[i]);
   }

   // wait for the threads to finish
   for (i = 0; i < nthreads; i++)
   {
      pthread_join(tid[i], NULL);
   }

   // sum up the individual results
   total_sum = 0;
   for (i = 0; i < nthreads; i++)
   {
      total_sum += sum[i];
      printf("sum[%d] = %d\n", i, sum[i]);
   }
   // compute our estimate of pi
   pi = 4.0 * (double)total_sum / (double)npoints;

   printf("Our estimate of pi = %.14f\n", pi);
   printf("                pi = %.14f\n", 4.0*atan(1.0));
   free(sum);
   return 0;
}//main()


void *thread(void *vargp)
{
   int my_rank =  *((int*)vargp);
   int my_sum = 0;
   int i;
   double rand_x;
   double rand_y;

   srand( (int)&i );

   for (i = 0; i < sample_points_per_thread; i++)
   {
      rand_x = 2*(rand()%10000)/10000.0 - 1.0; // random double between -1.0 and +1.0
      rand_y = 2*(rand()%10000)/10000.0 - 1.0; // random double between -1.0 and +1.0
      if ( (rand_x * rand_x) + (rand_y * rand_y) <= 1.0 )
      {
         my_sum++;
      }
   }
   sum[my_rank] = my_sum;  // this has false sharing (but it doesn't matter)
   //printf("my_rank = %d, my_sum = %d\n", my_rank, my_sum);
   return NULL;
}


/*
   Compute Pi using a "Monte Carlo" technique.
     http://en.wikipedia.org/wiki/Monte_Carlo_method

   The area of a circle of radius 1 is pi.
   If we randomly generate points from the
   unit square [-1, 1] x [-1, 1], with area 4,
   then each random point has probability pi/4
   of being in the unit circle centered at (0,0).

   If we generate a large number of such random points, then

(number of random pts in the circle) / (number of random pts generated)

   is an estimate of the probability that a randomly generated
   point was in the unit circle. So it is an estimate of pi/4.
   So 4 * that number is an estimate for pi.
*/