/*
   Compute Pi using a "Monte Carlo" technique.
      http://en.wikipedia.org/wiki/Monte_Carlo_method
   See the end of this file for some details.

   In this final version, we clean up the code, removing
   all traces of the original threaded code. In addition,
   we let OpenMP decide how many threads to create (this
   makes the program more portable and scalable).

   Notice how this program is essentially a single threaded
   program into which we have place a single OpenMP pragma!
*/
#include <stdio.h>
#include <windows.h>
#include <math.h>
#include <omp.h>

int main(int argc, char** argv)
{
   int i;
   double pi;
   int npoints;   // number of sample points to generate
   int sum;
   double rand_x;
   double rand_y;

   npoints = 100000000;  // 100,000,000

   srand( GetTickCount() );  // this is a Win32 function
   sum = 0;

   #pragma omp parallel for private(i) reduction(+: sum)// let OpenMP choose the number of threads
   for (i = 0; i < npoints; i++)  // let OpenMP divide up the loop among the threads
   {
      rand_x = 2*(rand()%10000)/10000.0 - 1.0; // random double between -1.0 and +1.0
      rand_y = 2*(rand()%10000)/10000.0 - 1.0; // random double between -1.0 and +1.0
      if ( (rand_x * rand_x) + (rand_y * rand_y) <= 1.0 )
      {
         sum++;  // let OpenMP handle the synchronization
      }
   }

   // compute our estimate of pi
   pi = 4.0 * (double)sum / (double)npoints;

   printf("Our estimate of pi = %.14f\n", pi);
   printf("                pi = %.14f\n", 4.0*atan(1.0));

   return 0;
}

/*
   Compute Pi using a "Monte Carlo" technique.
     http://en.wikipedia.org/wiki/Monte_Carlo_method

   The area of a circle of radius 1 is pi.
   If we randomly generate points from the
   unit square [-1, 1] x [-1, 1], with area 4,
   then each random point has probability pi/4
   of being in the unit circle centered at (0,0).

   If we generate a large number of such random points, then

(number of random pts in the circle) / (number of random pts generated)

   is an estimate of the probability that a randomly generated
   point was in the unit circle. So it is an estimate of pi/4.
   So 4 * that number is an estimate for pi.
*/